#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 14 11:46:11 2023

@author: eric
"""


from api import LensControllerAFZ
from api import LensControllerFocus
import serial.tools.list_ports
import logging
import time

def main():
#    logging.basicConfig(level=logging.ERROR)
    logger = logging.getLogger('logger')
    logger.setLevel(logging.INFO)
    # Now the logging.info messages will be printed
#    logging.info('This is an info message')
    logger.info('This is an info message')

    print("Available Serial Ports:")
    ports = list(serial.tools.list_ports.comports())
    for i, port in enumerate(ports):
        print(f"{i + 1}. {port}")
    print("0. Exit")

    if not ports:
        print("No available ports found.")
        return

    selected_port = int(input("Enter the port number you want to use: "))
    if selected_port == 0:
        return
    if selected_port < 0 or selected_port > len(ports):
        print("Invalid port number selected.")
        return
    selected_port -= 1
    
    print("Choose Controller type:")
    print("1. AFZ Controller")
    print("2. Focus Controller")
    selected_controller = int(input("Enter the controller number you want to use: "))

    if selected_controller == 2:
        lensController = LensControllerFocus(str(ports[selected_port].device))
        lensController.connect()   
        while True:

            print("\nFocus Controller tests")
            print("1. Get Focus position")
            print("2. Move Focus position")
            print("3. Stop Focus lens")
            print("4. Focus near")
            print("5. Focus far")
            print("6. Get end positions")
            print("7. Set focus speed (RPM)")
            print("8. Get focus speed (RPM)")
            print("9. Set focus position")
            print("0. Exit")
    
            choice = input("Enter your choice: ")
            if choice == "1":
                position, motor_moving = lensController.get_focus_position()
                print(f"Current position: {position}")
            elif choice == "2":
                while True:
                    try:
                        focus_value = input("Enter focus position (integer): ")
                        focus_value = int(focus_value)
                        print(f"Moving to position: {focus_value}")
                        break
                    except ValueError:  
                        print("Invalid input, please enter a number.")
                lensController.move_focus_position(focus_value)
            elif choice == "3":
                lensController.focus_stop()
            elif choice == "4":
                lensController.focus_near()
            elif choice == "5":
                lensController.focus_far()
            elif choice == "6":
                near, far = lensController.get_end_positions()
                print(f"Nearest position: {near}")
                print(f"Furthest position: {far}")
            elif choice == "7":
                while True:
                    try:
                        focus_speed = input("Enter focus speed in RPM: ")
                        focus_speed = int(focus_speed)
                        if focus_speed < 0 or focus_speed > 600:
                            print("Invalid input, please a value between 0 - 600")
                        else:
                            break
                    except ValueError:  
                        print("Invalid input, please enter a number.")
                lensController.set_focus_speed(focus_speed)
            elif choice == "8":
                focus_speed = lensController.get_focus_speed()
                print(f"Focus speed set to: {focus_speed} RPM")    
            elif choice == "9":
                while True:
                    try:
                        focus_position = input("Enter new focus position: ")
                        focus_position = int(focus_position)
                        if focus_position < -50000 or focus_position > 50000:
                            print("Invalid input, please a value between -500000 and 50000")
                        else:
                            break
                    except ValueError:  
                        print("Invalid input, please enter a number.")
                lensController.set_focus_position(focus_position)
            elif choice == "0":
                lensController.disconnect()
                break
            else:
                print("Invalid choice. Please enter a number or character.")
                
    if selected_controller == 1:
        lensController = LensControllerAFZ(str(ports[selected_port].device))
        lensController.connect()    
        while True:
            print("\nAFZ Lens Controller tests")
            print("-------- Custom commands --------")
            print("1. Select lens type")
            print("2. Get lens type")
            print("3. Calibrate all")
            print("4. get zoom position")
            print("5. set zoom position")
            print("6. get focus position")
            print("7. set focus position")
            print("8. get iris position")
            print("9. set iris position")
            print("10. Store current settings in lenscontroller")
            print("11. Set sensitivity")
            print("12. Get sensitivity")
            print("\n-------- Pelco D commands --------")
            print("20. Zoom In")
            print("21. Zoom Out")
            print("22. Focus Near")
            print("23. Focus Far")
            print("24. Iris Open")
            print("25. Iris Close")
            print("26. Stop all motors")
            print("27. Call Preset")
            print("28. Set Preset")
            print("29. Set Zoom Speed")
            print("30. Set Focus Speed")    
    
    
            print("0. Exit")
    
            choice = input("Enter your choice: ")
    
            if choice == "1":
                while True:
                    try:
                        print("\n1. Default AFZ lens")
                        print("2. AFZ2 lens")
                        lens_type = input("Enter lens type (integer): ")
                        lens_type = int(lens_type)
                        if lens_type < 0 or lens_type > 2:
                            print("Invalid input, please a value between 1 - 2.")
                        else:
                            break
                    except ValueError:
                        print("Invalid input, please enter a number.")
                if lens_type == 1:
                    lensController.custom_select_lens_default_afz() 
                elif lens_type == 2:
                    lensController.custom_select_lens_AFZ2()
                else:
                    print("Invalid lens type")
    
    
            elif choice == "2":
                lens_type = lensController.custom_get_lens_type()
                print(f"Selected lens type: {lens_type}")
            elif choice == "3":
                lensController.custom_calibrate_all()
            elif choice == "4": 
                position = lensController.custom_get_zoom_position()
                print(f"Current zoom position: {position}")            
            elif choice == "5": 
                while True:
                    try:
                        zoom_value = input("Enter zoom value (integer): ")
                        zoom_value = int(zoom_value)
                        if zoom_value < 0:
                            print("Invalid input, please enter a positive number.")
                        else:
                            break
                    except ValueError:
                        print("Invalid input, please enter a number.")
                lensController.custom_set_zoom_position(zoom_value)
            elif choice == "6": 
                position = lensController.custom_get_focus_position()
                print(f"Current focus position: {position}")            
            elif choice == "7": 
                while True:
                    try:
                        focus_value = input("Enter focus value (integer): ")
                        focus_value = int(focus_value)
                        if focus_value < 0:
                            print("Invalid input, please enter a positive number.")
                        else:
                            break
                    except ValueError:
                        print("Invalid input, please enter a number.")
                lensController.custom_set_focus_position(focus_value)
            elif choice == "8": 
                position = lensController.custom_get_iris_position()
                print(f"Current iris position: {position}")            
            elif choice == "9": 
                while True:
                    try:
                        iris_value = input("Enter iris value (integer): ")
                        iris_value = int(iris_value)
                        if iris_value < 0:
                            print("Invalid input, please enter a positive number.")
                        else:
                            break
                    except ValueError:
                        print("Invalid input, please enter a number.")
                lensController.custom_set_iris_position(iris_value)
            elif choice == "10":
                lensController.custom_store_current_settings()
                
            elif choice == "11": 
                while True:
                    try:
                        mode = input("Enter sensitivity mode value (integer): ")
                        mode = int(mode)
                        if mode < 0:
                            print("Invalid input, please enter a positive number.")
                        else:
                            break
                    except ValueError:
                        print("Invalid input, please enter a number.")
                lensController.custom_set_sensitivity_mode(mode)
            elif choice == "12":
                sensitivity_mode = lensController.custom_get_sensitivity_mode()            
                print(f"sensitivity mode: {sensitivity_mode}")       
    
            elif choice == "20":
                lensController.pelco_d_zoom_in()
            elif choice == "21":
                lensController.pelco_d_zoom_out()
            elif choice == "22":
                lensController.pelco_d_focus_near()
            elif choice == "23":
                lensController.pelco_d_focus_far()
            elif choice == "24":
                lensController.pelco_d_iris_open()
            elif choice == "25":
                lensController.pelco_d_iris_close()
            elif choice == "26":
                lensController.pelco_d_stop_all()
                
            elif choice == "27":
                preset = int(input("Enter preset number to call [1-10]: "))
                lensController.pelco_d_call_preset(preset)
            elif choice == "28":
                preset = int(input("Enter preset number to set [1-10]: "))
                lensController.pelco_d_set_preset(preset)
            elif choice == "29":
                speed = int(input("Enter zoom speed [0-2]: "))
                lensController.pelco_d_set_zoom_speed(speed)
            elif choice == "30":
                speed = int(input("Enter focus speed [0-2]: "))
                lensController.pelco_d_set_focus_speed(speed)    
        
            elif choice == "0":
                lensController.disconnect()
                break
            else:
                print("Invalid choice. Please enter a number or character.")

if __name__ == "__main__":
    main()